@@ -16,8 +16,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include "LogBookConfigDialog.hpp"
+#include "LogBook.hpp"
 #include "LogBookCommon.hpp"
+#include "LogBookConfigDialog.hpp"
 #include "dataMappers/BarlowsDataMapper.hpp"
 #include "dataMappers/FiltersDataMapper.hpp"
 #include "dataMappers/ImagersDataMapper.hpp"
@@ -27,7 +28,11 @@
 #include "dataMappers/SitesDataMapper.hpp"
 
 #include "StelApp.hpp"
+#include "StelGui.hpp"
 #include "StelMainGraphicsView.hpp"
+#include "StelModuleMgr.hpp"
+#include "StelStyle.hpp"
+#include "StelTranslator.hpp"
 
 #include "ui_LogBookConfigDialog.h"
 
@@ -90,12 +95,21 @@ void LogBookConfigDialog::languageChanged()
 	}
 }
 
-
 void LogBookConfigDialog::styleChanged()
 {
 	// Nothing for now
 }
 
+void LogBookConfigDialog::updateStyle()
+{
+	if(dialog) {
+		StelGui* gui = dynamic_cast<StelGui*>(StelApp::getInstance().getGui());
+		Q_ASSERT(gui);
+		const StelStyle pluginStyle = GETSTELMODULE(LogBook)->getModuleStyleSheet(gui->getStelStyle());
+		dialog->setStyleSheet(pluginStyle.qtStyleSheet);
+	}
+}
+
 /* ********************************************************************* */
 #if 0
 #pragma mark -
@@ -120,6 +134,9 @@ void LogBookConfigDialog::createDialogContent()
 	connect(ui->closeStelWindow, SIGNAL(clicked()), this, SLOT(close()));
 	setupWidgets();
 	setupListViews();
+
+	//Initialize the style
+	updateStyle();
 }
 
 void LogBookConfigDialog::setupListViews()
@@ -208,13 +225,13 @@ void LogBookConfigDialog::setupWidgets()
 	widgets[SITES] = sites;
 	
 	// Layout the ListWidget
-	ui->pagesListWidget->addItem(tr("Barlows"));
-	ui->pagesListWidget->addItem(tr("Filters"));
-	ui->pagesListWidget->addItem(tr("Imagers"));
-	ui->pagesListWidget->addItem(tr("Observers"));
-	ui->pagesListWidget->addItem(tr("Oculars"));
-	ui->pagesListWidget->addItem(tr("Optics"));
-	ui->pagesListWidget->addItem(tr("Sites"));
+	ui->pagesListWidget->addItem(N_("Barlows"));
+	ui->pagesListWidget->addItem(N_("Filters"));
+	ui->pagesListWidget->addItem(N_("Imagers"));
+	ui->pagesListWidget->addItem(N_("Observers"));
+	ui->pagesListWidget->addItem(N_("Oculars"));
+	ui->pagesListWidget->addItem(N_("Optics"));
+	ui->pagesListWidget->addItem(N_("Sites"));
 	ui->pagesListWidget->setCurrentRow(0);
 	
 	connect(ui->pagesListWidget, SIGNAL(currentRowChanged(int)), ui->stackedWidget, SLOT(setCurrentIndex(int)));
