@@ -39,7 +39,9 @@ Vec3d StelObject::getEquinoxEquatorialPos(const StelCore* core) const
 // Get observer local sideral coordinate
 Vec3d StelObject::getSideralPosGeometric(const StelCore* core) const
 {
-	return Mat4d::zrotation(-core->getLocalSideralTime())* getEquinoxEquatorialPos(core);
+	// Hour Angle corrected to Delta-T value
+	double dt = (core->getDeltaT(core->getJDay())/240.)*M_PI/180.;
+	return Mat4d::zrotation(-core->getLocalSideralTime()+dt)* getEquinoxEquatorialPos(core);
 }
 
 // Get observer local sidereal coordinates, deflected by refraction
@@ -47,7 +49,9 @@ Vec3d StelObject::getSideralPosApparent(const StelCore* core) const
 {
 	Vec3d v=getAltAzPosApparent(core);
 	v = core->altAzToEquinoxEqu(v, StelCore::RefractionOff);
-	return Mat4d::zrotation(-core->getLocalSideralTime())*v;
+	// Hour Angle corrected to Delta-T value
+	double dt = (core->getDeltaT(core->getJDay())/240.)*M_PI/180.;
+	return Mat4d::zrotation(-core->getLocalSideralTime()+dt)*v;
 }
 
 Vec3d StelObject::getAltAzPosGeometric(const StelCore* core) const
@@ -103,7 +107,8 @@ QString StelObject::getPositionInfoString(const StelCore *core, const InfoString
 	{
 		double glong, glat;
 		StelUtils::rectToSphe(&glong, &glat, getJ2000GalacticPos(core));
-		res += q_("Galactic longitude/latitude (J2000): %1/%2").arg(StelUtils::radToDmsStr(glong,true), StelUtils::radToDmsStr(glat,true)) + "<br>";
+		// Note that Gal. Coords are DEFINED in B1950 coordinates, and writing "J2000" to them does not make any sense.
+		res += q_("Galactic longitude/latitude: %1/%2").arg(StelUtils::radToDmsStr(glong,true), StelUtils::radToDmsStr(glat,true)) + "<br>";
 	}
 
 	if (flags&HourAngle)
