@@ -67,11 +67,19 @@ void StelTranslator::initSystemLanguage(void)
 		else
 		{
 #ifdef Q_OS_WIN
-			char cc[3];
-			if (GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SISO639LANGNAME, cc, 3))
+			char ulng[3], ctry[3];
+			if (GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SISO639LANGNAME, ulng, 3))
 			{
-				cc[2] = '\0';
-				systemLangName = cc;
+				ulng[2] = '\0';
+				if (GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SISO3166CTRYNAME, ctry, 3))
+				{
+					ctry[2] = '\0';
+					systemLangName = QString("%1_%2").arg(ulng).arg(ctry);
+				}
+				else
+				{
+					systemLangName = ulng;
+				}
 			}
 			else
 			{
