@@ -101,6 +101,13 @@ public:
 	//! @param maxNbItem the maximum number of returned object names.
 	//! @return a list of matching object name by order of relevance, or an empty list if nothing matches.
 	virtual QStringList listMatchingObjectsI18n(const QString& objPrefix, int maxNbItem=5) const;
+	//! Find objects by translated name prefix.
+	//! Find and return the list of at most maxNbItem objects auto-completing
+	//! the passed object English name.
+	//! @param objPrefix the case insensitive first letters of the searched object.
+	//! @param maxNbItem the maximum number of returned object names.
+	//! @return a list of matching object name by order of relevance, or an empty list if nothing matches.
+	virtual QStringList listMatchingObjects(const QString& objPrefix, int maxNbItem=5) const;
 	virtual QStringList listAllObjects(bool inEnglish) const;
 	virtual QString getName() const { return "Solar System"; }
 
