@@ -58,6 +58,8 @@ private:
 	QStringList values;
 };
 
+QT_FORWARD_DECLARE_CLASS(QListWidgetItem)
+
 //! @class SearchDialog
 //! The sky object search dialog.
 class SearchDialog : public StelDialog
@@ -91,6 +93,11 @@ private slots:
 	void onSearchTextChanged(const QString& text);
 	
 	void gotoObject();
+	void gotoObject(const QString& nameI18n);
+	// for going from list views
+	void gotoObject(QListWidgetItem* item);
+
+	void searchListChanged(const QString& newText);
 	
 	//! Called when the user edit the manual position controls
 	void manualPositionChanged();
@@ -104,6 +111,12 @@ private slots:
 	//! Called when a SIMBAD server is selected in the list.
 	void selectSimbadServer(int index);
 
+	//! Called when new type of objects selected in list view tab
+	void updateListWidget(int index);
+
+	// retranslate/recreate tab
+	void updateListTab();
+
 private:
 	class SimbadSearcher* simbadSearcher;
 	class SimbadLookupReply* simbadReply;
