@@ -582,11 +582,16 @@ void BottomStelBar::updateText(bool updatePos)
 	StelCore* core = StelApp::getInstance().getCore();
 	double jd = core->getJDay();
 	double deltaT = 0.;
+	double sigma = -1.;
+	QString sigmaInfo = "";
+	QString validRangeInfo = "";
 	bool displayDeltaT = false;
-	if (StelApp::getInstance().getCore()->getCurrentLocation().planetName=="Earth")
+	if (core->getCurrentLocation().planetName.contains("Earth"))
 	{
-		deltaT = StelUtils::getDeltaT(jd);
-		displayDeltaT = true;
+		deltaT = core->getDeltaT(jd);
+		displayDeltaT = true;		
+		sigma = StelUtils::getDeltaTStandardError(jd);
+		core->getCurrentDeltaTAlgorithmValidRange(jd, &validRangeInfo);
 	}
 
 	// Add in a DeltaT correction. Divide DeltaT by 86400 to convert from seconds to days.
@@ -596,12 +601,15 @@ void BottomStelBar::updateText(bool updatePos)
 	{
 		updatePos = true;
 		datetime->setText(newDate);
-		if (displayDeltaT)
+		if (displayDeltaT && core->getCurrentDeltaTAlgorithm()!=StelCore::WithoutCorrection)
 		{
-			if (deltaT>60.)
-				datetime->setToolTip(QString("%1T = %2 (%3s)").arg(QChar(0x0394)).arg(StelUtils::hoursToHmsStr(deltaT/3600.)).arg(deltaT, 5, 'f', 2));
+			if (sigma>0)
+				sigmaInfo = QString("; %1(%2T) = %3s").arg(QChar(0x03c3)).arg(QChar(0x0394)).arg(sigma, 3, 'f', 1);
+
+			if (std::abs(deltaT)>60.)
+				datetime->setToolTip(QString("%1T = %2 (%3s)%8 [%4 @ -23.8946%5/cy%6%7]").arg(QChar(0x0394)).arg(StelUtils::hoursToHmsStr(deltaT/3600.)).arg(deltaT, 5, 'f', 2).arg(QChar(0x1e45)).arg(QChar(0x2033)).arg(QChar(0x00B2)).arg(sigmaInfo).arg(validRangeInfo));
 			else
-				datetime->setToolTip(QString("%1T = %2s").arg(QChar(0x0394)).arg(deltaT, 3, 'f', 3));
+				datetime->setToolTip(QString("%1T = %2s%7 [%3 @ -23.8946%4/cy%5%6]").arg(QChar(0x0394)).arg(deltaT, 3, 'f', 3).arg(QChar(0x1e45)).arg(QChar(0x2033)).arg(QChar(0x00B2)).arg(sigmaInfo).arg(validRangeInfo));
 		}
 		else
 			datetime->setToolTip("");
@@ -615,6 +623,20 @@ void BottomStelBar::updateText(bool updatePos)
 	{
 		updatePos = true;
 		location->setText(newLocation);
+		float lat = core->getCurrentLocation().latitude;
+		float lon = core->getCurrentLocation().longitude;
+		QString latStr, lonStr, pm;
+		if (lat >= 0)
+			pm = "N";
+		else
+			pm = "S";
+		latStr = QString("%1%2%3").arg(pm).arg(lat).arg(QChar(0x00B0));
+		if (lat >= 0)
+			pm = "E";
+		else
+			pm = "W";
+		lonStr = QString("%1%2%3").arg(pm).arg(lon).arg(QChar(0x00B0));
+		location->setToolTip(QString("%1 %2").arg(latStr).arg(lonStr));
 	}
 
 	QSettings* confSettings = StelApp::getInstance().getSettings();
@@ -719,10 +741,11 @@ void BottomStelBar::buttonHoverChanged(bool b)
 	Q_ASSERT(button);
 	if (b==true)
 	{
-		if (button->action)
+		QAction* action = button->action;
+		if (action)
 		{
-			QString tip(button->action->toolTip());
-			QString shortcut(button->action->shortcut().toString());
+			QString tip(action->toolTip());
+			QString shortcut(action->shortcut().toString(QKeySequence::NativeText));
 			if (!shortcut.isEmpty())
 			{
 				if (shortcut == "Space")
