@@ -19,8 +19,9 @@ rmdir ${CMAKE_INSTALL_PREFIX}/Resources/stellarium
 
 mkdir ${CMAKE_INSTALL_PREFIX}/Frameworks
 /usr/bin/perl util/pkgApp.pl ${CMAKE_INSTALL_PREFIX} MacOS/stellarium Frameworks 
-cp -pr /Developer/Applications/Qt/plugins/{imageformats,iconengines} ${CMAKE_INSTALL_PREFIX}/MacOS
-for f in ${CMAKE_INSTALL_PREFIX}/MacOS/{imageformats,iconengines}/*.dylib; do
+mkdir ${CMAKE_INSTALL_PREFIX}/plugins
+cp -pr /Developer/Applications/Qt/plugins/{imageformats,iconengines}/* ${CMAKE_INSTALL_PREFIX}/plugins
+for f in ${CMAKE_INSTALL_PREFIX}/plugins/*.dylib; do
     fdir=`dirname $f`
     dir=`basename $fdir`
     base=`basename $f`
